/*
 * PROJECT: matereal at http://mr.digitalmuseum.jp/
 * ----------------------------------------------------------------------------
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is matereal.
 *
 * The Initial Developer of the Original Code is Jun KATO.
 * Portions created by the Initial Developer are
 * Copyright (C) 2009 Jun KATO. All Rights Reserved.
 *
 * Contributor(s): Jun KATO
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */
package jp.digitalmuseum.mr.core;

import jp.digitalmuseum.connector.Connector;

/**
 * Abstract implementation of Resource.<br />
 * Resource implementation classes must extend this abstract class.
 *
 * @author Jun KATO
 */
public abstract class ResourceAbstractImpl implements Resource {
	private RobotAbstractImpl robot;
	private Connector connector;

	protected ResourceAbstractImpl(RobotAbstractImpl robot) {
		this.robot = robot;
		setConnector(robot.getConnector());
	}

	protected ResourceAbstractImpl(Connector connector) {
		setConnector(connector);
	}

	/**
	 * Set a connector for communication with the robot to this resource.
	 *
	 * @param connector
	 */
	protected void setConnector(Connector connector) {
		this.connector = connector;
		if (connector != null && !connector.isConnected()) {
			connector.connect();
		}
	}

	/**
	 * Called when disposing this resource.<br />
	 * <b>Caution:</b> This method should not be called outside this class. Use free() instead.
	 *
	 * @see #free()
	 */
	abstract protected void dispose();

	/**
	 * @return Returns a robot to which this resource belongs.
	 */
	final protected RobotAbstractImpl getRobot() {
		return robot;
	}

	/**
	 * @return Returns a connector.
	 * @see #setConnector(Connector)
	 */
	final protected Connector getConnector() {
		return connector;
	}

	/**
	 * Called when finished using this resource.
	 */
	final public void free() {
		dispose();
		if (robot != null) {
			robot.freeResource(this);
		}
	}

}
