/*
 * PROJECT: matereal at http://mr.digitalmuseum.jp/
 * ----------------------------------------------------------------------------
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is matereal.
 *
 * The Initial Developer of the Original Code is Jun KATO.
 * Portions created by the Initial Developer are
 * Copyright (C) 2009 Jun KATO. All Rights Reserved.
 *
 * Contributor(s): Jun KATO
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */
package jp.digitalmuseum.mr.entity;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import jp.digitalmuseum.mr.core.Behavior;
import jp.digitalmuseum.mr.core.Resource;
import jp.digitalmuseum.mr.core.Robot;
import jp.digitalmuseum.mr.core.ProxyResourceAbstractImpl;
import jp.digitalmuseum.mr.core.ProxyRobotAbstractImpl;
import jp.digitalmuseum.mr.resource.Wheels;

/**
 * ProxyRobot: CloneWheelsRobot<br />
 * Proxy robot class that copies moving functions to multiple robots.
 *
 * @author Jun KATO
 */
public class CloneWheelsRobot extends ProxyRobotAbstractImpl {
	private Set<Robot> robots;
	private Set<Wheels> wheels;
	private Wheels w;

	public CloneWheelsRobot(Set<Robot> robots) {
		this.robots = new HashSet<Robot>(robots);
		wheels = new HashSet<Wheels>();
		w = new CloneWheels(this);
	}

	@Override
	protected Resource[] getResources() {
		return new Resource[] { w };
	}

	public Set<Robot> getRelatedRobots() {
		return new HashSet<Robot>(robots);
	}

	@Override
	public synchronized HashMap<Class<? extends Resource>, Resource>
			requestResources(Behavior behavior, Set<Class<? extends Resource>> resourceTypes) {
		if (resourceTypes.size() != 1 ||
				!resourceTypes.contains(Wheels.class)) {
			return null;
		}
		HashMap<Class<? extends Resource>, Resource> virtualResourceMap = super.requestResources(behavior, resourceTypes);
		if (virtualResourceMap != null) {
			for (Robot robot : robots) {
				final HashMap<Class<? extends Resource>, Resource> resourceMap =
						robot.requestResources(behavior, resourceTypes);
				if (resourceMap == null) {
					virtualResourceMap = null;
					break;
				}
				wheels.add((Wheels) resourceMap.get(Wheels.class));
			}

			// Free all resources when failed to acquire all resources.
			if (virtualResourceMap == null) {
				for (Wheels w : wheels) {
					w.free();
				}
			}
		}
		return virtualResourceMap;
	}

	@Override
	protected synchronized void freeResource(Resource resource) {
		for (Wheels w : wheels) {
			w.free();
		}
		super.freeResource(resource);
	}

	private class CloneWheels extends ProxyResourceAbstractImpl implements Wheels {

		public CloneWheels(ProxyRobotAbstractImpl robot) {
			super(robot);
		}

		public void goBackward() {
			for (Wheels wh : wheels) { wh.goBackward(); }
		}
		public void goForward() {
			for (Wheels wh : wheels) { wh.goForward(); }
		}
		public void spin(SPIN direction) {
			for (Wheels wh : wheels) { wh.spin(direction); }
		}
		public void stopWheels() {
			for (Wheels wh : wheels) { wh.stopWheels(); }
		}
		public void dispose() {
			stopWheels();
		}
	}
}
