package sample;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JFrame;

import jp.digitalmuseum.mr.Matereal;
import jp.digitalmuseum.mr.entity.MindstormsNXT;
import jp.digitalmuseum.mr.entity.Robot;
import jp.digitalmuseum.mr.gui.DisposeOnCloseFrame;
import jp.digitalmuseum.mr.gui.ImageProviderPanel;
import jp.digitalmuseum.mr.service.MarkerDetector;
import jp.digitalmuseum.mr.service.Camera;
import jp.digitalmuseum.mr.task.Rotate;
import jp.digitalmuseum.mr.task.Task;
import jp.digitalmuseum.napkit.NapMarker;
import jp.digitalmuseum.napkit.gui.TypicalMDCPane;
import jp.digitalmuseum.utils.Position;
import jp.digitalmuseum.utils.ScreenPosition;
//import jp.digitalmuseum.mr.entity.Noopy;


/**
 * Show a frame with which an end-user can specify in which direction a robot should face.
 *
 * @author Jun KATO
 */
public class Step4_ClickAndRotate {
	private Task workingTask;
	final private ScreenPosition clickedPosition = new ScreenPosition();

	public static void main(String[] args) {
		new Step4_ClickAndRotate();
	}

	public Step4_ClickAndRotate() {

		final Robot robot = new MindstormsNXT("Mindstorms NXT", "btspp://00165306523e");

		// Run a camera.
		final Camera camera = new Camera();
		camera.start();

		// Run a marker detector.
		final MarkerDetector detector = new MarkerDetector();
		detector.put(new NapMarker("markers/4x4_78.patt", 4.5), robot);
		detector.start();

		// Show a configuration window.
		final JFrame configFrame = new DisposeOnCloseFrame(
				new TypicalMDCPane(detector));

		// Initialize a main panel.
		final ImageProviderPanel panel = new ImageProviderPanel(camera) {
			private static final long serialVersionUID = 1L;
			@Override public void paintComponent(Graphics g) {
				super.paintComponent(g);
				final Graphics2D g2 = (Graphics2D) g;
				g2.translate(getOffsetX(), getOffsetY());

				// Paint detection results.
				g2.setStroke(new BasicStroke(4));
				g2.setColor(Color.orange);
				detector.paint(g2);

				// Paint the destination.
				if (clickedPosition != null) {
					g2.setColor(Color.green);
					g2.fillOval(
							clickedPosition.getX()-5,
							clickedPosition.getY()-5,
							10, 10);
				}
			}
			@Override public void dispose() {
				configFrame.dispose();
				super.dispose();
				Matereal.getInstance().dispose();
			}
		};

		// Add a mouse listener.
		panel.addMouseListener(new MouseAdapter() {
			final private Position destination =
				new Position();
			private Position position =
				new Position();
			@Override public void mouseClicked(MouseEvent e) {
				if (e.getButton() == MouseEvent.BUTTON1) {

					// Get a clicked position.
					clickedPosition.set(e.getX(), e.getY());
					panel.getScreenToImageOut(clickedPosition, clickedPosition);
					camera.screenToRealOut(clickedPosition, destination);
					detector.getPositionOut(robot, position);

					final double destinationDirection =
						position.getRelativeDirection(destination);

					// If another task is already working, stop it.
					if (workingTask != null && workingTask.isStarted()) {
						workingTask.stop();
					}

					// Assign a task rotating to a certain direction.
					workingTask = new Rotate(destinationDirection);
					if (workingTask.assign(robot)) {
						workingTask.start();
					}
				}
			}
		});

		// Show a world.
		final DisposeOnCloseFrame frame = new DisposeOnCloseFrame(panel);
		frame.setFrameSize(560, 420);
	}
}
