package sample.test.pen.logic;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;

import sample.test.pen.DrawPath;
import sample.test.pen.Path;

import jp.digitalmuseum.mr.core.TransitionAbstractImpl;
import jp.digitalmuseum.mr.core.service.CoordProvider;
import jp.digitalmuseum.mr.core.task.Task;
import jp.digitalmuseum.mr.core.task.LocationBasedTask;
import jp.digitalmuseum.mr.task.EndPen;
import jp.digitalmuseum.mr.task.Move;
import jp.digitalmuseum.mr.task.PutPen;
import jp.digitalmuseum.mr.task.RotateTo;
import jp.digitalmuseum.utils.Position;
import jp.digitalmuseum.utils.ScreenPosition;
import jp.digitalmuseum.utils.Vector2D;

public class DrawPath_4_LazyGoalRefined extends DrawPath {

	final private static double DUMMY_LENGTH = 10.0;
	final private static double GOAL_WIDTH = 5.0;

	final private Path path;
	private Task bootstrap;
	private BufferedImage layer;

	public DrawPath_4_LazyGoalRefined(Path path) {
		this.path = path;
	}

	protected void onStart() {
		super.onStart();

		final CoordProvider coordProvider = getLocationProvider().getCoordProvider();
		final Task putPen = new PutPen();

		layer = new BufferedImage(
				coordProvider.getWidth(), coordProvider.getHeight(),
				BufferedImage.TYPE_INT_ARGB);
		final Graphics2D g2 = layer.createGraphics();
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setColor(Color.red);

		final Position destination = new Position();
		final Position departure = new Position();
		final Position dummy = new Position();
		final Position p = new Position();
		final Vector2D e = new Vector2D();

		Task task = putPen;

		for (final ScreenPosition sp : path) {

			coordProvider.screenToRealOut(sp, destination);

			if (bootstrap == null) {
				bootstrap = new Move(destination);
				final RotateTo rotateTo = new RotateTo(destination);
				bootstrap.setDefaultTransition(rotateTo);
				rotateTo.setDefaultTransition(task);
			}

			else {

				// Get unit vector from departure to destination.
				e.set(destination);
				e.sub(departure);
				e.div(e.getNorm());

				// Calculate dummy destination.
				dummy.set(destination);
				e.mul(DUMMY_LENGTH);
				dummy.add(e);
				e.div(e.getNorm());

				// Move to the dummy destination.
				final Move moveTo = new Move(dummy);
				moveTo.addTransition(
						new LineOverDetection(destination, e));
				task.setDefaultTransition(moveTo);
				task = moveTo;

				// Paint the transition line.
				final double
						dx = -e.getY()*GOAL_WIDTH/2,
						dy = e.getX()*GOAL_WIDTH/2;

				p.set(destination);
				p.add(dx, dy);
				coordProvider.realToScreenOut(p, sp);

				final int sx = sp.getX(), sy = sp.getY();
				p.set(destination);
				p.add(-dx, -dy);
				coordProvider.realToScreenOut(p, sp);

				g2.drawLine(sx, sy, sp.getX(), sp.getY());
			}

			departure.set(destination);
		}

		final Task endPen = new EndPen();
		task.setDefaultTransition(endPen);
		endPen.setDefaultTransition(this);

		task = putPen;
		g2.dispose();
	}

	@Override
	protected void behave() {
		if (bootstrap == null) {
			fireDefaultTransition();
		} else {
			delegateToTask(bootstrap);
			bootstrap = null;
		}
	}

	@Override
	public void paint(Graphics2D g) {
		super.paint(g);
		g.drawImage(layer, 0, 0, null);
	}

	private static class LineOverDetection extends TransitionAbstractImpl {
		final Position p = new Position();
		final Vector2D e;
		final Position destination;

		public LineOverDetection(Position destination, Vector2D e) {
			this.destination = new Position(destination);
			this.e = new Vector2D(e);
		}

		/**
		 * Transit to next task when the robot is going over the destination.
		 */
		@Override
		public boolean check(Task task) {
			((LocationBasedTask) task).getPositionOut(p);
			p.sub(destination);
			return p.dot(e) > 0;
		}
	}
}
