package sample;

import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JFrame;

import jp.digitalmuseum.mr.behavior.Move;
import jp.digitalmuseum.mr.behavior.Stop;
import jp.digitalmuseum.mr.core.Behavior;
import jp.digitalmuseum.mr.core.Matereal;
import jp.digitalmuseum.mr.core.Robot;
import jp.digitalmuseum.mr.entity.MindstormsNXT;
import jp.digitalmuseum.mr.gui.*;
import jp.digitalmuseum.mr.resource.WheelsController;
import jp.digitalmuseum.mr.service.Camera;
import jp.digitalmuseum.mr.service.MarkerDetector;
import jp.digitalmuseum.napkit.NapMarker;
import jp.digitalmuseum.napkit.gui.TypicalMDCPane;
import jp.digitalmuseum.utils.ScreenPosition;

/**
 * Click and run!
 *
 * @author Jun KATO
 */
public class ClickAndRun {
	private Move moveTo;
	final private ScreenPosition goalPosition = new ScreenPosition();

	public static void main(String[] args) {
		new ClickAndRun();
	}

	public ClickAndRun() {

		// Initialize a robot.
		final Robot robot = new MindstormsNXT("Mindstorms NXT", "btspp://00165305b308");

		// Run a camera.
		final Camera camera = new Camera();
		camera.start();
		Matereal.getInstance().setInterval(camera.getActualInterval());

		// Run a marker detector.
		final MarkerDetector detector = new MarkerDetector();
		detector.setImageProvider(camera);
		detector.put(new NapMarker("markers/4x4_743.patt", 12), robot);
		detector.start();

		// Show a configuration window.
		final JFrame configFrame = new DisposeOnCloseFrame(
				new TypicalMDCPane(detector));

		// Make and show a window for showing captured image.
		final DrawableFrame frame = new DrawableFrame() {
			private static final long serialVersionUID = 1L;

			@Override public void dispose() {
				super.dispose();
				configFrame.dispose();
				Matereal.getInstance().dispose();
			}

			@Override
			public void paint2D(Graphics2D g) {
				camera.drawImage(g);
				final Behavior behavior = robot.getAssignedBehavior(WheelsController.class);
				if (behavior != null) {
					g.drawString("Status: "+behavior, 10, 30);
					g.drawLine(
							goalPosition.getX()-5,
							goalPosition.getY()-5,
							goalPosition.getX()+5,
							goalPosition.getY()+5);
					g.drawLine(
							goalPosition.getX()-5,
							goalPosition.getY()+5,
							goalPosition.getX()+5,
							goalPosition.getY()-5);
				}
				else {
					g.drawString("Status: Goal not specified", 10, 30);
				}
				g.drawLine(10, 35, 175, 35);
			}
		};
		frame.setResizable(false);
		frame.setFrameSize(camera.getWidth(), camera.getHeight());

		// Go to the clicked location when clicked.
		frame.getPanel().addMouseListener(new MouseAdapter() {
			@Override
			public void mouseReleased(MouseEvent e) {
				goalPosition.set(e.getX(), e.getY());
				final Behavior behavior = robot.getAssignedBehavior(WheelsController.class);
				if (behavior != null) {
					behavior.stop();
				}
				moveTo = new Move(camera.screenToReal(goalPosition));
				moveTo.setDefaultTransition(new Stop());
				if (moveTo.assignToRobot(robot)) {
					moveTo.setSpeed(10);
					moveTo.setRotationSpeed(5);
					moveTo.start();
				}
			}
		});

		// Repaint the window periodically.
		Matereal.getInstance().addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) { frame.repaint(); }
		});
	}
}
