package sample;

import java.awt.BasicStroke;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.util.Set;
import javax.swing.JFrame;

import jp.digitalmuseum.mr.core.Entity;
import jp.digitalmuseum.mr.core.EntityImpl;
import jp.digitalmuseum.mr.core.Matereal;
import jp.digitalmuseum.mr.gui.DisposeOnCloseFrame;
import jp.digitalmuseum.mr.gui.ImageProviderPanel;
import jp.digitalmuseum.mr.service.MarkerDetector;
import jp.digitalmuseum.mr.service.Camera;
import jp.digitalmuseum.napkit.NapDetectionResult;
import jp.digitalmuseum.napkit.NapMarker;
import jp.digitalmuseum.napkit.gui.TypicalMDCPane;
import jp.digitalmuseum.utils.ScreenPosition;

/**
 * Run a marker detection and show results.
 *
 * @author Jun KATO
 */
public class Step2_RunDetectMarker {

	public static void main(String[] args) {
		new Step2_RunDetectMarker();
	}

	public Step2_RunDetectMarker() {

		// Run a camera.
		final Camera camera = new Camera();
		camera.start();
		Matereal.getInstance().setInterval(camera.getActualInterval());

		// Run a marker detector.
		final MarkerDetector detector = new MarkerDetector();
		detector.loadCameraParameter("calib_qcam.dat");
		detector.start();

		// Show a configuration window.
		final JFrame configFrame = new DisposeOnCloseFrame(new TypicalMDCPane(detector));

		// Detect a marker.
		final NapMarker marker = new NapMarker("markers\\4x4_743.patt",120);
		final Entity dummy = new EntityImpl("test");
		detector.put(marker, dummy);

		// Show detection results in real-time.
		final DisposeOnCloseFrame frame = new DisposeOnCloseFrame(
				new ImageProviderPanel(camera) {
					private static final long serialVersionUID = 1L;
					private transient Stroke stroke;
					@Override public void paintComponent(Graphics g) {
						super.paintComponent(g);
						final Graphics2D g2 = (Graphics2D) g;
						if (stroke == null) {
							stroke = new BasicStroke(5);
						}
						g2.setStroke(stroke);
						g2.translate(getOffsetX(), getOffsetY());

						// Get detected results.
						final Set<NapDetectionResult> results = detector.getResults();

						// Draw each detected result.
						for (final NapDetectionResult result : results) {

							// Draw corners
							g.setColor(Color.orange);
							detector.paint(g2, result);

							// Draw information for the square.
							g.setColor(Color.cyan);
							final ScreenPosition point = result.getPosition();
							g2.drawLine(point.getX(), point.getY(), point.getX()+55, point.getY()+43);
							g2.drawRect(point.getX()+55, point.getY()+23, 200, 40);
							drawString(g2, "Confidence: "+result.getConfidence(), point.getX()+64, point.getY()+40);
							drawString(g2, "Position: "+point, point.getX()+64, point.getY()+56);
						}

						// Draw detected number of squares.
						drawString(g2, "detected: "+(results == null ? 0 : results.size()),
								10, getHeight()-10);
					}

					/**
					 * Draw a string with 1px simple black border.
					 */
					private void drawString(Graphics g, String s, int x, int y) {
						g.setColor(Color.black);
						g.drawString(s, x+1, y);
						g.drawString(s, x-1, y);
						g.drawString(s, x, y-1);
						g.drawString(s, x, y+1);
						g.setColor(Color.cyan);
						g.drawString(s, x, y);
					}
				}) {
			private static final long serialVersionUID = 1L;
			@Override public void dispose() {
				configFrame.dispose();
				super.dispose();
				Matereal.getInstance().dispose();
			}
		};
		frame.setFrameSize(camera.getWidth(), camera.getHeight());
	}
}
