package sample;

import java.awt.BasicStroke;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JFrame;

import jp.digitalmuseum.mr.behavior.Move;
import jp.digitalmuseum.mr.behavior.Stop;
import jp.digitalmuseum.mr.core.Behavior;
import jp.digitalmuseum.mr.core.Matereal;
import jp.digitalmuseum.mr.core.Robot;
import jp.digitalmuseum.mr.entity.MindstormsNXT;
import jp.digitalmuseum.mr.gui.DisposeOnCloseFrame;
import jp.digitalmuseum.mr.gui.ImageProviderPanel;
import jp.digitalmuseum.mr.resource.WheelsController;
import jp.digitalmuseum.mr.service.MarkerDetector;
import jp.digitalmuseum.mr.service.Camera;
import jp.digitalmuseum.napkit.NapMarker;
import jp.digitalmuseum.napkit.gui.TypicalMDCPane;
import jp.digitalmuseum.utils.Position;
import jp.digitalmuseum.utils.ScreenPosition;

/**
 * Show a frame with a view of the world coordinate.
 *
 * @author Jun KATO
 */
public class Step4_ClickAndRun {
	final private ScreenPosition clickedPosition =
		new ScreenPosition();

	public static void main(String[] args) {
		new Step4_ClickAndRun();
	}

	public Step4_ClickAndRun() {

		// Destination is not specified at first.
		clickedPosition.setNotFound(true);

		// Initialize a robot.
		final Robot robot = new MindstormsNXT("Mindstorms NXT", "btspp://00165305b308");

		// Run a camera.
		final Camera camera = new Camera();
		camera.start();
		Matereal.getInstance().setInterval(camera.getActualInterval());

		// Run a marker detector.
		final MarkerDetector detector = new MarkerDetector();
		detector.setImageProvider(camera);
		detector.put(new NapMarker("markers/4x4_743.patt", 12), robot);
		detector.start();

		// Show a configuration window.
		final JFrame configFrame = new DisposeOnCloseFrame(
				new TypicalMDCPane(detector));

		// Initialize a main panel.
		final ImageProviderPanel panel = new ImageProviderPanel(camera) {
			private static final long serialVersionUID = 1L;
			@Override public void paintComponent(Graphics g) {
				super.paintComponent(g);
				final Graphics2D g2 = (Graphics2D) g;
				g2.translate(getOffsetX(), getOffsetY());

				// Paint detection results.
				g2.setStroke(new BasicStroke(4));
				g2.setColor(Color.orange);
				detector.paint(g2);

				// Paint the destination.
				if (!clickedPosition.isNotFound()) {
					g2.setColor(Color.green);
					g2.fillOval(
							clickedPosition.getX()-5,
							clickedPosition.getY()-5,
							10, 10);
				}
			}
			@Override public void dispose() {
				configFrame.dispose();
				super.dispose();
				Matereal.getInstance().dispose();
			}
		};

		// Add a mouse listener.
		panel.addMouseListener(new MouseAdapter() {
			final private Position destination =
				new Position();

			@Override public void mouseClicked(MouseEvent e) {
				if (e.getButton() == MouseEvent.BUTTON1) {

					// Get a clicked position.
					clickedPosition.set(e.getX(), e.getY());
					panel.getScreenToImageOut(clickedPosition, clickedPosition);
					camera.screenToRealOut(clickedPosition, destination);

					// Assign a behavior moving to the position.
					final Behavior currentBehavior = robot.getAssignedBehavior(WheelsController.class);
					if (currentBehavior != null) {
						currentBehavior.stop();
					}
					final Move moveTo = new Move(destination);
					moveTo.setDefaultTransition(new Stop());
					if (moveTo.assignToRobot(robot)) {
						moveTo.setSpeed(10);
						moveTo.setRotationSpeed(5);
						moveTo.start();
					}
				}
			}
		});

		// Show a world.
		final JFrame frame = new DisposeOnCloseFrame(panel);
		frame.setSize(710, 660);
	}
}
