package sample.test;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;

import sample.test.pen.Path;

import jp.digitalmuseum.mr.core.Matereal;
import jp.digitalmuseum.mr.gui.DrawableFrame;

/**
 * Draw strokes on a canvas.
 *
 * @author Jun KATO
 */
public class StrokePainter {
	private Path currentPath;
	private ArrayList<Path> paths;

	public static void main(String[] args) {
		new StrokePainter();
	}

	public StrokePainter() {
		super();

		paths = new ArrayList<Path>();

		final DrawableFrame drawableFrame = new DrawableFrame() {
			private static final long serialVersionUID = 1L;

			@Override
			public void dispose() {
				super.dispose();
				Matereal.getInstance().dispose();
			}
			public void paint2D(Graphics2D g) {
				g.setColor(Color.white);
				g.fillRect(0, 0, getWidth(), getHeight());
				g.setColor(Color.black);
				for (Path path : paths) {
					path.paint(g);
				}
				if (currentPath != null) {
					g.setColor(Color.red);
					currentPath.paint(g);
				}
			}
		};
		drawableFrame.setFrameSize(320, 240);

		drawableFrame.getPanel().addMouseListener(new MouseListener() {
			private boolean in = false;
			public void mousePressed(MouseEvent e) {
				if (in) {
					currentPath = new Path();
				}
			}
			public void mouseReleased(MouseEvent e) {
				if (currentPath.size() > 0) {
					currentPath.resample(10);
					paths.add(currentPath);
					currentPath = null;
					drawableFrame.getPanel().repaint();
				}
			}
			public void mouseEntered(MouseEvent e) {
				in = true;
			}
			public void mouseExited(MouseEvent e) {
				in = false;
			}
			public void mouseClicked(MouseEvent e) {
			}
		});

		drawableFrame.getPanel().addMouseMotionListener(new MouseMotionListener(){
			public void mouseDragged(MouseEvent e) {
				currentPath.add(e.getX(), e.getY());
				drawableFrame.getPanel().repaint();
			}
			public void mouseMoved(MouseEvent e) {
			}
		});
	}
}
