package sample.test.pen;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Stroke;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JCheckBox;

import sample.test.pen.logic.DrawPath_2_AsLines;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;

import jp.digitalmuseum.mr.behavior.Stop;
import jp.digitalmuseum.mr.core.Behavior;
import jp.digitalmuseum.mr.core.Robot;
import jp.digitalmuseum.mr.gui.ImageProviderPanel;
import jp.digitalmuseum.mr.resource.WheelsController;
import jp.digitalmuseum.mr.service.ImageProvider;
import javax.swing.JToolBar;
import javax.swing.JButton;

public class StrokePlayerPanel extends JPanel implements WizardComponent {

	private Class<? extends DrawPath> logic = DrawPath_2_AsLines.class;
	private static final long serialVersionUID = 1L;
	private JPanel strokePlayerPanel = null;
	private JCheckBox jStrokeCheckBox = null;
	private transient ImageProvider imageProvider;
	private transient PathsProvider pathProvider;
	private transient LogicSelector logicSelector;
	private transient Robot robot;
	private transient Stroke stroke;  //  @jve:decl-index=0:
	private transient List<Path> paths;
	private transient List<DrawPath> drawPaths;
	private transient boolean isShowingIdealPaths = true;
	private transient AlphaComposite alphaComp = AlphaComposite.getInstance(
			AlphaComposite.SRC_OVER, 0.3f);

	/**
	 * This is the default constructor
	 */
	public StrokePlayerPanel(Class<? extends DrawPath> logic, ImageProvider imageProvider) {
		super();
		this.logic = logic;
		this.imageProvider = imageProvider;
		initialize();
	}

	public void setPathsProvider(PathsProvider pathProvider) {
		this.pathProvider = pathProvider;
	}

	public PathsProvider getPathsProvider() {
		return pathProvider;
	}

	public void setLogicSelector(LogicSelector logicSelector) {
		this.logicSelector = logicSelector;
	}

	public LogicSelector getLogicSelector() {
		return logicSelector;
	}

	public void setRobot(Robot robot) {
		this.robot = robot;
	}

	public Robot getRobot() {
		return robot;
	}

	public void setStroke(Stroke stroke) {
		this.stroke = stroke;
	}

	public Stroke getStroke() {
		return stroke;
	}

	public void setShowingIdealPaths(boolean isShowingIdealPaths) {
		this.isShowingIdealPaths = isShowingIdealPaths;
		getJStrokeCheckBox().setSelected(isShowingIdealPaths);
	}

	public boolean isShowingIdealPaths() {
		return isShowingIdealPaths;
	}

	public void onLoad() {
		final Behavior assignedBehavior =
			robot.getAssignedBehavior(WheelsController.class);
		if (assignedBehavior != null) {
			if (assignedBehavior.isPaused()) {
				assignedBehavior.resume();
			}
		} else {
			play();
		}
	}

	public void onUnload() {
		final Behavior assignedBehavior =
			robot.getAssignedBehavior(WheelsController.class);
		if (assignedBehavior != null) {
			assignedBehavior.pause();
		}
	}

	private void play() {
		synchronized (pathProvider) {
			drawPaths = new ArrayList<DrawPath>();
			DrawPath bootstrap = null;
			DrawPath drawPath = null;
			paths = pathProvider.getPaths();
			for (Path path : paths) {
				if (drawPath == null) {
					drawPath = new_DrawPath(path);
					bootstrap = drawPath;
				} else {
					final DrawPath drawNextPath = new_DrawPath(path);
					drawPath.setDefaultTransition(drawNextPath);
					drawPath = drawNextPath;
				}
				drawPaths.add(drawPath);
			}
			if (bootstrap != null && bootstrap.assignToRobot(robot)) {
				bootstrap.setSpeed(10);
				bootstrap.setRotationSpeed(5);
				bootstrap.start();
				drawPath.setDefaultTransition(new Stop());
			}
		}
	}

	private void saveTrack() {
		final JFileChooser filechooser = new JFileChooser();

		int selected = filechooser.showSaveDialog(getParent());
		if (selected == JFileChooser.APPROVE_OPTION) {
			File file = filechooser.getSelectedFile();
			BufferedImage image = new BufferedImage(
					imageProvider.getWidth(),
					imageProvider.getHeight(),
					BufferedImage.TYPE_INT_ARGB);
			final Graphics2D g = image.createGraphics();
			for (DrawPath drawPath : drawPaths) {
				final BufferedImage img = drawPath.getImage();
				if (img != null) {
					g.drawImage(img, 0, 0, null);
				}
			}
			g.dispose();
			try {
				ImageIO.write(image, "PNG", file);
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}
	}

	private void saveImage() {
		final JFileChooser filechooser = new JFileChooser();

		int selected = filechooser.showSaveDialog(getParent());
		if (selected == JFileChooser.APPROVE_OPTION) {
			File file = filechooser.getSelectedFile();
			BufferedImage image = new BufferedImage(
					imageProvider.getWidth(),
					imageProvider.getHeight(),
					BufferedImage.TYPE_INT_ARGB);
			final Graphics2D g = image.createGraphics();
			imageProvider.drawImage(g);
			paintStatus(g);
			g.dispose();
			try {
				ImageIO.write(image, "PNG", file);
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}
	}

	private Constructor<? extends DrawPath> constructor;
	private JToolBar jToolBar = null;
	private JButton jSaveTrackButton = null;
	private JButton jSaveImageButton = null;
	private DrawPath new_DrawPath(Path path) {
		if (logicSelector != null &&
				logicSelector.getSelectedLogic() != null) {
			logic = logicSelector.getSelectedLogic();
		}
		try {
			if (constructor == null ||
					!constructor.getDeclaringClass().equals(logic)) {
				constructor = logic.getConstructor(Path.class);
			}
			return constructor.newInstance(path);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * This method initializes this
	 *
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setLayout(new BorderLayout());
		this.add(getStrokePlayerPanel(), BorderLayout.CENTER);
		this.add(getJStrokeCheckBox(), BorderLayout.NORTH);
		this.add(getJToolBar(), BorderLayout.SOUTH);
	}

	/**
	 * This method initializes strokePlayerPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getStrokePlayerPanel() {
		if (strokePlayerPanel == null) {
			strokePlayerPanel = new ImageProviderPanel(imageProvider) {
				private static final long serialVersionUID = 1L;

				@Override
				public void paintComponent(Graphics g) {
					super.paintComponent(g);
					Graphics2D g2 = (Graphics2D)g;
					g2.translate(getOffsetX(), getOffsetY());
					paintStatus(g2);
				}
			};
			strokePlayerPanel.addMouseListener(new MouseAdapter() {

				@Override
				public void mouseClicked(MouseEvent e) {
					saveImage();
				}
			});
		}
		return strokePlayerPanel;
	}

	private void paintStatus(Graphics2D g) {
		g.setRenderingHint(
				RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		// Paint path.
		if (isShowingIdealPaths() && paths != null) {
			final Stroke originalStroke = g.getStroke();
			if (stroke != null) {
				g.setStroke(stroke);
			}
			g.setColor(Color.black);
			for (Path path : paths) {
				path.paint(g);
			}
			g.setStroke(originalStroke);
		}

		// Paint status of the robot.
		Behavior assignedBehavior = robot.getAssignedBehavior(WheelsController.class);
		if (assignedBehavior != null) {
			g.drawString(assignedBehavior.toString(), 5, 30);
			if (assignedBehavior instanceof DrawPath) {
				g.setColor(Color.blue);
				g.setComposite(alphaComp);
				((DrawPath) assignedBehavior).paint(g);
			}
		}
	}

	/**
	 * This method initializes jStrokeCheckBox
	 *
	 * @return javax.swing.JCheckBox
	 */
	private JCheckBox getJStrokeCheckBox() {
		if (jStrokeCheckBox == null) {
			jStrokeCheckBox = new JCheckBox();
			jStrokeCheckBox.setText("Show ideal paths (original strokes)");
			jStrokeCheckBox.setSelected(isShowingIdealPaths);
			jStrokeCheckBox.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {
					setShowingIdealPaths(jStrokeCheckBox.isSelected());
				}
			});
		}
		return jStrokeCheckBox;
	}

	/**
	 * This method initializes jToolBar
	 *
	 * @return javax.swing.JToolBar
	 */
	private JToolBar getJToolBar() {
		if (jToolBar == null) {
			jToolBar = new JToolBar();
			jToolBar.add(getJSaveTrackButton());
			jToolBar.add(getJSaveImageButton());
		}
		return jToolBar;
	}

	/**
	 * This method initializes jSaveTrackButton
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getJSaveTrackButton() {
		if (jSaveTrackButton == null) {
			jSaveTrackButton = new JButton();
			jSaveTrackButton.setText("Save track of the robot");
			jSaveTrackButton.setToolTipText("Save track of the robot as an image file.");
			jSaveTrackButton.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent arg0) {
					saveTrack();
				}
			});
		}
		return jSaveTrackButton;
	}

	/**
	 * This method initializes jSaveImageButton
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getJSaveImageButton() {
		if (jSaveImageButton == null) {
			jSaveImageButton = new JButton();
			jSaveImageButton.setText("Capture current status");
			jSaveImageButton.setToolTipText("Capture current status and save as an image file.");
			jSaveImageButton.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					saveImage();
				}
			});
		}
		return jSaveImageButton;
	}

}
