package sample.test.pen;

import java.awt.BasicStroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;

import sample.test.pen.logic.*;

import jp.digitalmuseum.connector.ConnectorFactory;
import jp.digitalmuseum.mr.core.Matereal;
import jp.digitalmuseum.mr.core.Robot;
import jp.digitalmuseum.mr.entity.HakoniwaRobotWithPen;
import jp.digitalmuseum.mr.entity.HumanBeing;
import jp.digitalmuseum.mr.entity.HumanBeingWithPen;
import jp.digitalmuseum.mr.entity.MindstormsNXTWithPen;
import jp.digitalmuseum.mr.gui.DisposeOnCloseFrame;
import jp.digitalmuseum.mr.gui.DrawableFrame;
import jp.digitalmuseum.mr.gui.HumanBeingFrame;
import jp.digitalmuseum.mr.gui.ResourceViewer;
import jp.digitalmuseum.mr.resource.Wheels;
import jp.digitalmuseum.mr.service.Camera;
import jp.digitalmuseum.mr.service.Hakoniwa;
import jp.digitalmuseum.mr.service.MarkerDetector;
import jp.digitalmuseum.napkit.NapMarker;
import jp.digitalmuseum.napkit.gui.TypicalMDCPane;
import jp.digitalmuseum.utils.Position;

public class YouRobotMain {
	public final static boolean DEBUG = false;
	private final static Class<? extends DrawPath> DEFAULT_LOGIC = DrawPath_4_LazyGoalRefined.class;
	private Robot robot;

	public static void main(String[] args) {
		new YouRobotMain();
	}

	public YouRobotMain() {

		final WizardFrame wizardFrame = new WizardFrame(
				DEBUG ? startVirtual() : startReal());

		// Make a window for showing status of the wheels.
		final Wheels wheels = robot.requestResource(Wheels.class, null);
		final DisposeOnCloseFrame wheelsViewerFrame = new DisposeOnCloseFrame(
				new ResourceViewer(wheels));

		wizardFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		wizardFrame.setSize(740, 680);
		wizardFrame.setVisible(true);
		wizardFrame.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosed(WindowEvent e) {
				wheelsViewerFrame.dispose();
				Matereal.getInstance().dispose();
			}
		});
		wizardFrame.setTitle("youRobot");
	}

	private JComponent[] startVirtual() {

		// Run hakoniwa.
		final Hakoniwa hakoniwa = new Hakoniwa();
		hakoniwa.setAntialiased(true);
		hakoniwa.setBackgroundTransparent(false);
		hakoniwa.setViewportSize(640, 480);
		hakoniwa.start();

		// Instantiate a robot.
		robot = new HakoniwaRobotWithPen("test", new Position(
				hakoniwa.getRealWidth()/2,
				hakoniwa.getRealHeight()/2));

		// Initialize wizard components.
		final StrokePainterPanel strokePainterPanel = new StrokePainterPanel(hakoniwa);
		strokePainterPanel.setStroke(new BasicStroke(3));
		final LogicSelectorPanel logicSelectorPanel = new LogicSelectorPanel();
		final StrokePlayerPanel strokePlayerPanel = new StrokePlayerPanel(DEFAULT_LOGIC, hakoniwa);
		strokePlayerPanel.setPathsProvider(strokePainterPanel);
		strokePlayerPanel.setLogicSelector(logicSelectorPanel);
		strokePlayerPanel.setStroke(strokePainterPanel.getStroke());
		strokePlayerPanel.setRobot(robot);

		// Show a wizard.
		final JComponent[] components = new JComponent[] {
			strokePainterPanel,
			logicSelectorPanel,
			strokePlayerPanel
		};
		components[0].setName("Draw exemplar paths");
		components[1].setName("Select logic");
		components[2].setName("Let the robot play");
		return components;
	}

	private JComponent[] startReal() {

		// Instantiate a robot.
		/*
		robot = new HumanBeingWithPen();
		final DrawableFrame humanBeingFrame =
				new HumanBeingFrame((HumanBeing) robot);
		humanBeingFrame.setFrameSize(320, 320);
		Matereal.getInstance().addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				humanBeingFrame.repaint();
			}
		});
		*/

		///* --- Comment out here to use the real robot. *

		robot = new MindstormsNXTWithPen("MindstormsNXT",
				ConnectorFactory.makeConnector("btspp://00165305b308"));

		// * --------------------------------------- --- */

		// Run a camera.
		final Camera camera = new Camera();
		camera.start();
		Matereal.getInstance().setInterval(camera.getActualInterval());

		// Run a marker detector.
		final MarkerDetector detector = new MarkerDetector();
		detector.loadCameraParameter("calib_qcam.dat");
		detector.start();

		// Detect a marker.
		//final NapMarker marker = new NapMarker("markers\\4x4_368.patt",4.5);
		final NapMarker marker = new NapMarker("markers\\4x4_190.patt",120);
		detector.put(marker, robot);

		// Initialize wizard components.
		final StrokePainterPanel strokePainterPanel = new StrokePainterPanel(camera);
		strokePainterPanel.setStroke(new BasicStroke(3));
		final LogicSelectorPanel logicSelectorPanel = new LogicSelectorPanel();
		final StrokePlayerPanel strokePlayerPanel = new StrokePlayerPanel(DEFAULT_LOGIC, camera);
		strokePlayerPanel.setPathsProvider(strokePainterPanel);
		strokePlayerPanel.setLogicSelector(logicSelectorPanel);
		strokePlayerPanel.setStroke(strokePainterPanel.getStroke());
		strokePlayerPanel.setRobot(robot);

		// Show a wizard.
		final JComponent[] components = new JComponent[] {
			new TypicalMDCPane(detector),
			strokePainterPanel,
			logicSelectorPanel,
			strokePlayerPanel
		};
		components[0].setName("Setup marker detection");
		components[2].setName("Draw exemplar paths");
		components[1].setName("Select logic");
		components[3].setName("Let the robot play");
		return components;
	}
}
