package sample.test.pen.logic;

import java.awt.Graphics2D;
import java.util.Set;

import jp.digitalmuseum.mr.behavior.LocationBasedBehaviorAbstractImpl;
import jp.digitalmuseum.mr.behavior.Move;
import jp.digitalmuseum.mr.behavior.RotateTo;
import jp.digitalmuseum.mr.behavior.VectorBehavior;
import jp.digitalmuseum.mr.core.Behavior;
import jp.digitalmuseum.mr.core.Resource;
import jp.digitalmuseum.mr.gui.utils.VectorFieldPainter;
import jp.digitalmuseum.mr.resource.PenController;
import jp.digitalmuseum.mr.service.CoordProvider;
import jp.digitalmuseum.utils.Position;

public class DrawLine extends LocationBasedBehaviorAbstractImpl {
	private Position departure;
	private Position destination;
	private int step;
	private boolean ok = false;

	public DrawLine(Position departure, Position destination) {
		this.departure = new Position(departure);
		this.destination = new Position(destination);
		step = 0;
	}

	@Override
	public String getName() {
		return "Draw a line from "+departure+" to "+destination
		/*+(hasDelegate() ? ", "+getDelegatedBehavior().toString() : "")*/;
	}

	@Override
	public Set<Class<? extends Resource>> getRequirements() {
		final Set<Class<? extends Resource>> resourceTypes = super.getRequirements();
		resourceTypes.add(PenController.class);
		return resourceTypes;
	}

	@Override
	public void onAssigned() {
		super.onAssigned();
		PenController pen = (PenController) getResourceMap().get(PenController.class);
		pen.putPen(); // Make sure that the pen is down.
	}

	@Override
	protected void behave() {
		switch (step) {
		case 0:
			Behavior moveToDeparture = new Move(departure);
			moveToDeparture.setDefaultTransition(this);
			if (delegateToBehavior(moveToDeparture)) {
				step ++;
			}
			break;
		case 1:
			Behavior rotateTo = new RotateTo(getPosition().getRelativeDirection(destination));
			rotateTo.setDefaultTransition(this);
			if (delegateToBehavior(rotateTo)) {
				step ++;
			}
			break;
		case 2:
			Behavior moveToDestination = new Move(destination);
			moveToDestination.setDefaultTransition(this);
			if (delegateToBehavior(moveToDestination)) {
				step ++;
			}
			break;
		default:
			ok = true;
			break;
		}
	}

	public void paint(VectorFieldPainter vectorFieldPainter, Graphics2D g) {
		if (hasDelegate()) {
			final Behavior behavior = getDelegationChild();
			if (behavior instanceof VectorBehavior) {
				final VectorBehavior vectorBehavior =
						(VectorBehavior) behavior;
				final CoordProvider coordProvider =
						vectorBehavior.getLocationProvider().getCoordProvider();
				vectorFieldPainter.paint(coordProvider, vectorBehavior, g);
			}
		}
	}

	@Override
	protected boolean checkDefaultTransition() {
		return ok;
	}
}
