package sample.test.pen.logic;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.util.Iterator;

import sample.test.pen.DrawPath;
import sample.test.pen.Path;

import jp.digitalmuseum.mr.behavior.EndPen;
import jp.digitalmuseum.mr.behavior.Move;
import jp.digitalmuseum.mr.behavior.PutPen;
import jp.digitalmuseum.mr.core.Behavior;
import jp.digitalmuseum.mr.core.TransitionAbstractImpl;
import jp.digitalmuseum.mr.resource.Pen.STATUS;
import jp.digitalmuseum.mr.service.CoordProvider;
import jp.digitalmuseum.utils.Position;
import jp.digitalmuseum.utils.ScreenPosition;
import jp.digitalmuseum.utils.Vector2D;

public class DrawPath_3_LazyGoal extends DrawPath {
	private Iterator<ScreenPosition> pathIterator;
	private Position departure;
	private Position destination;

	final private static double DUMMY_LENGTH = 20.0;
	final private static double GOAL_WIDTH = 150.0;
	final Vector2D e = new Vector2D();
	final private Position p = new Position();
	final private ScreenPosition sp = new ScreenPosition();

	public DrawPath_3_LazyGoal(Path path) {
		this.pathIterator = path.iterator();
		destination = new Position();
		departure = new Position();
		departure.setNotFound(true);
	}

	@Override
	protected void behave() {
		if (pathIterator.hasNext()) {
			if (!departure.isNotFound() && getPen().getStatus() == STATUS.UP) {
				putPen();
			}
			else {
				visitNextPoint();
			}
		}
		else if (getPen().getStatus() == STATUS.DOWN) {
			endPen();
		}
		else {
			fireDefaultTransition();
		}
	}

	private void putPen() {
		PutPen putPen = new PutPen();
		putPen.setDefaultTransition(this);
		delegateToBehavior(putPen);
	}

	private void endPen() {
		EndPen endPen = new EndPen();
		endPen.setDefaultTransition(this);
		delegateToBehavior(endPen);
	}

	private void visitNextPoint() {

		final ScreenPosition nextScreenPosition = pathIterator.next();
		getLocationProvider().getCoordProvider().screenToRealOut(
				nextScreenPosition, destination);
		if (departure.isNotFound()) {
			Move moveTo = new Move(destination);
			moveTo.setDefaultTransition(this);
			delegateToBehavior(moveTo);
		}
		else {
			// Get unit vector from departure to destination.
			e.set(destination);
			e.sub(departure);
			e.div(e.getNorm());

			// Calculate dummy destination.
			final Position dummyDestination = new Position(destination);
			e.mul(DUMMY_LENGTH);
			dummyDestination.add(e);
			e.div(e.getNorm());

			// Assign behavior to move to the dummy destination.
			Move drawLine = new Move(dummyDestination);
			drawLine.setDefaultTransition(this);
			drawLine.addTransition(new TransitionAbstractImpl(this) {
				final Position p = new Position();

				/**
				 * Transit to next behavior when the robot is going over the destination.
				 */
				@Override
				public boolean check(Behavior behavior) {
					getPositionOut(p);
					p.sub(destination);
					return p.dot(e) > 0;
				}
			});
			delegateToBehavior(drawLine);
		}
		departure.set(destination);
	}

	@Override
	public void paint(Graphics2D g) {
		super.paint(g);
		if (hasDelegate()) {
			final CoordProvider coordProvider =
					getLocationProvider().getCoordProvider();

			// Save graphics preference.
			final Color color = g.getColor();
			final Composite comp = g.getComposite();
			g.setColor(Color.red);
			g.setComposite(AlphaComposite.SrcOver);

			// Draw transition goal.
			if (!destination.isNotFound()) {
				final double
						dx = -e.getY()*GOAL_WIDTH/2,
						dy = e.getX()*GOAL_WIDTH/2;

				p.set(destination);
				p.add(dx, dy);
				coordProvider.realToScreenOut(p, sp);
				final int sx = sp.getX(), sy = sp.getY();

				p.set(destination);
				p.add(-dx, -dy);
				coordProvider.realToScreenOut(p, sp);

				g.drawLine(sx, sy, sp.getX(), sp.getY());
			}

			// Restore graphics preference.
			g.setColor(color);
			g.setComposite(comp);
		}
	}

}
