/*
 * PROJECT: napkit at http://matereal.sourceforge.jp/
 * ----------------------------------------------------------------------------
 *
 * This file is part of NyARToolkit Application Toolkit.
 * NyARToolkit Application Toolkit, or simply "napkit",
 * is a simple wrapper library for NyARToolkit.
 *
 * ----------------------------------------------------------------------------
 *
 * License version: GPL 3.0
 *
 * napkit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * napkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with napkit. If not, see <http://www.gnu.org/licenses/>.
 */
package jp.digitalmuseum.napkit;

import java.util.Set;

import jp.digitalmuseum.utils.ScreenRectangle;
import jp.nyatla.nyartoolkit.NyARException;
import jp.nyatla.nyartoolkit.core.match.NyARMatchPattDeviationColorData;
import jp.nyatla.nyartoolkit.core.match.NyARMatchPattResult;
import jp.nyatla.nyartoolkit.core.squaredetect.NyARSquare;


/**
 * Simple marker matching strategy.
 *
 * @author Jun KATO
 * @see NapMarkerDetector
 * @see NapMarkerMatchingStrategy
 */
public class NapSimpleMMStrategy implements NapMarkerMatchingStrategy {
	protected final NyARMatchPattResult matchResult
			= new NyARMatchPattResult();

	public NapDetectionResult getBestMatch(
			NyARMatchPattDeviationColorData deviationData,
			Set<NapMarker> markers,
			ScreenRectangle rectangle,
			long session) throws NyARException {

		// Prepare temporary result holder.
		NapMarker foundMarker = null;
		double confidence = 0;
		int direction = NyARSquare.DIRECTION_UNKNOWN;

		// Compare with all markers.
		for (NapMarker marker : markers) {

			// Evaluate.
			marker.getPattern().evaluate(deviationData, matchResult);

			// Get confidence.
			final double currentConfidence = matchResult.confidence;
			if (confidence > currentConfidence) {
				continue;
			}

			// Got better result.
			confidence = currentConfidence;
			direction = matchResult.direction;
			foundMarker = marker;
		}
		return new NapDetectionResult(foundMarker, rectangle, confidence, direction);
	}

}
